use std::{cell::RefCell, path::PathBuf, rc::Rc};

pub mod parser;
use parser::block::CommandBlock;
use parser::command::Command;
use parser::event::Event;
use parser::parse_file;

pub struct State {
    script: Rc<RefCell<CommandBlock>>,
}

impl State {
    pub fn from_file(file: PathBuf) -> State {
        State {
            script: parse_file(&file),
        }
    }

    pub fn next(&mut self) -> Option<Event> {
        while let Some(command) = self.next_command() {
            let event = command.execute(&self.script);
            if event.as_ref().map_or(false, |event| event.is_blocking()) {
                return event;
            }
        }
        None
    }

    fn next_command(&mut self) -> Option<Command> {
        self.script.borrow_mut().next()
    }
}
